SimpleAuraFilter = LibStub("AceAddon-3.0"):NewAddon("SimpleAuraFilter", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("SimpleAuraFilter")

S = SimpleAuraFilter

SimpleAuraFilter.debug = true


function SimpleAuraFilter:OnInitialize()
    -- Called when the addon is loaded
   
end



function SimpleAuraFilter:OnEnable()
    -- Called when the addon is enabled
    self.db = LibStub("AceDB-3.0"):New("SimpleAuraFilterDB")
    if not self.db.char.filters then self.db.char.filters = {} end
	
	
	local options = {
		name = "SimpleAuraFilter",
		handler = SimpleAuraFilter,
		type = 'group',
		args = {
			menu = {
				type = 'execute',
				name = 'Buff Filter Menu',
				desc = 'Shows filter list',
				func = 'OpenMenu',
			},
			
			toggle = {
				type = 'execute',
				name = 'Toggle Filter',
				desc = 'toggles buffs',
				func = 'ToggleAllBuffs',
			},
		},
	}
	
	options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)


	
	
	self.buffs = {}
    
	--Hooking the neccesary functions here
	self.BuffButton_UpdateAnchors_Old = BuffButton_UpdateAnchors
	BuffButton_UpdateAnchors = function (buttonName, N) return self:BuffButton_UpdateAnchors(buttonName, N) end
	
	self.BuffFrame_Update_Old = BuffFrame_Update
	BuffFrame_Update = function (b) return self:BuffFrame_Update() end
	
	BuffButton_OnLoad = function (button) return SimpleAuraFilter:BuffButton_OnLoad(button) end
	BuffButton_OnClick = function (button) return SimpleAuraFilter:BuffButton_OnClick(button) end
	
	self.AuraButton_Update_Old = AuraButton_Update
	AuraButton_Update = function(buttonName, index, filter) return self:AuraButton_Update(buttonName, index, filter) end
	
	
	-- Chat Command
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SimpleAuraFilter", options, {"saf"})
end



function SimpleAuraFilter:OnDisable()

end

-- ********* Hooks

function SimpleAuraFilter:BuffFrame_Update()
	self.buffs = {}
	-- Handle Buffs
	BUFF_ACTUAL_DISPLAY = 0;
	for i=1, BUFF_MAX_DISPLAY do
		self.buffs[i] = BUFF_ACTUAL_DISPLAY + 1
		if ( AuraButton_Update("BuffButton", i, "HELPFUL") ) then
			BUFF_ACTUAL_DISPLAY = BUFF_ACTUAL_DISPLAY + 1;
		end
	end
 
	-- Handle debuffs
	DEBUFF_ACTUAL_DISPLAY = 0;
	for i=1, DEBUFF_MAX_DISPLAY do
		if ( AuraButton_Update("DebuffButton", i, "HARMFUL") ) then
			DEBUFF_ACTUAL_DISPLAY = DEBUFF_ACTUAL_DISPLAY + 1;
		end
	end
end


function SimpleAuraFilter:BuffButton_OnLoad (button)
	button:RegisterForClicks("RightButtonUp");
end
 
function SimpleAuraFilter:BuffButton_OnClick (button)
	if IsShiftKeyDown() then
		name = UnitAura("player", button:GetID(), button.filter)
		if name then
			self.db.char.filters[name] = 1
			BuffFrame_Update()
		end
	else
		CancelUnitBuff(button.unit, button:GetID(), button.filter);
	end
end



function SimpleAuraFilter:BuffButton_UpdateAnchors(buttonName, index)
	local position = self.buffs[index]
	
	local function find(position)
		local v = 1
		while self.buffs[v] do
			if self.buffs[v] > position then return v-1 end
			v = v + 1
		end
		return nil
	end
	
	if not position then return end
	local buff = getglobal(buttonName..index);
	
 
	if ( (position > 1) and (mod(position, BUFFS_PER_ROW) == 1) ) then
		-- New row
		if ( position == BUFFS_PER_ROW+1 ) then
			buff:ClearAllPoints()
			buff:SetPoint("TOP", TempEnchant1, "BOTTOM", 0, -BUFF_ROW_SPACING);
		else
			buff:ClearAllPoints()
			buff:SetPoint("TOP", getglobal(buttonName..(find(position-BUFFS_PER_ROW))), "BOTTOM", 0, -BUFF_ROW_SPACING);
		end
	elseif ( position == 1 ) then
		buff:ClearAllPoints()
		buff:SetPoint("TOPRIGHT", BuffFrame, "TOPRIGHT", 0, 0);
	else
		buff:ClearAllPoints()
		buff:SetPoint("RIGHT", getglobal(buttonName..(find(position-1))), "LEFT", -5, 0);
	end
end




function SimpleAuraFilter:AuraButton_Update(buttonName, index, filter)
	if not (filter == "HELPFUL") then
		return self.AuraButton_Update_Old(buttonName, index, filter) 
	end

	if self:AllBuffs() then
		return self.AuraButton_Update_Old(buttonName, index, filter) 
	else
		name = UnitAura("player", index, filter)
		if self:IsBadBuff(name) then
		-- copied from BuffFrame.lua
			local buffName = buttonName..index;
			local buff = getglobal(buffName);
			local buffDuration = getglobal(buffName.."Duration");
			if ( buff ) then
				buff:Hide();
				buffDuration:Hide();
			end
		-- snap
			return nil
		else
			return self.AuraButton_Update_Old(buttonName, index, filter)
		end
	end
end



-- *********


function SimpleAuraFilter:IsBadBuff(name)
	return self.db.char.filters[name]
end


function SimpleAuraFilter:AllBuffs()
	return self.allbuffs
end

function SimpleAuraFilter:ToggleAllBuffs()
	self.allbuffs = not self.allbuffs
	BuffFrame_Update()
end


function SimpleAuraFilter:OpenMenu()
    local d = LibStub("AceGUI-3.0"):Create("Frame")
	d:SetTitle("Filters")
    d:SetWidth(400)
    d:SetHeight(225)
	d:SetLayout("Fill")
	local s = LibStub("AceGUI-3.0"):Create("ScrollFrame")
	--s:SetLayout("Float")
	d:AddChild(s)
	
	
	for name,_ in pairs(self.db.char.filters) do
		local temp = LibStub("AceGUI-3.0"):Create("Button")				
		temp:SetText(name)
		temp:SetCallback("OnClick", function (self, event)
						SimpleAuraFilter.db.char.filters[name] = nil
						d:Hide()
						BuffFrame_Update()
						SimpleAuraFilter:OpenMenu()
						end)
		s:AddChild(temp)
	end
    d:Show()
end


-- ********* Helpers

function SimpleAuraFilter:Debug(...)
    if self.debug then self:Print(...) end
end